#!/bin/sh

### Tunable Parameters
#

CRICKETDAEMON_IP_ADDR=10.0.0.1
#CRICKETDAEMON_IP_ADDR=127.0.0.1

#
# This defines the maximum beacon-listener separation distance during
# calibration.  That is, the listener must be held within SEPDIST_CM cm
# directly below a beacon during calibration.  When SEPDIST_CM is large
# relative to the distances between beacons (1.5x or greater), the user
# should move swiftly from one beacon to another during the calibration
# process.  Otherwise, BeaconConfig might start a measurement while 
# the listener is not directly underneath a beacon and cause errors
# in the beacon coordinate computation.
# 

SEPDIST_CM=5000

#
# STDDEV_TOLERANCE_CM is a threshold value expressed in (cm)
# This value is used to define
#
# 1) The threshold for determining a stable distance measurement during
#    the calibration phase.  A distance measurement is "stable" when 
#    the stddev value of the distances collected over 
#    the last FILTER_WINDOW_MS ms falls below STDDEV_TOLERANCE_CM.
#
#
# The recommended value for Cricket v1 is 15 and for Cricket v2 10 cm.
#

STDDEV_TOLERANCE_CM=10

#
# The size of the sample monitor history window (ms) and the max value 
# for the y-axis (cm).
#

SAMPLEMONHISTORY_MS=5000
SAMPLEMONMAXY_CM=400

#
# Uncommenting the following will enable Sun's Java HotSpot Server mode.
# The jvm will perform dynamic compiling to increase performance.
# However, the start up time will be quite slow.
# In general, the applications will run sluggishly in the first minute or
# so (depending on the computer speed) until the compilation is
# complete.  After the first minute, the CPU load should drop
# significantly.
#

JAVAHOTSPOT_ENABLE="-server"

#
#
############################################# End of Tunable Params


if [ "$OS" = "Windows_NT" ]
then SEP=";"
else SEP=":"
fi

LIB=""
#LIB="../lib/"

CLASSPATH=".${SEP}${LIB}plot.jar${SEP}samplemon.jar${SEP}beaconfinder.jar${SEP}beaconconfig.jar${SEP}cricketdaemon.jar${SEP}${LIB}Jama-1.0.1.jar${SEP}viewfinder.jar${SEP}${LIB}gnu.getopt.jar"

if [ "$MODE" = "simulation" ]
then 
echo ">>>>>>>>>>> Simulation Mode";
CRICKETDAEMON_IP_ADDR=127.0.0.1
fi

## Launch BeaconFinderApp
java $JAVAHOTSPOT -cp $CLASSPATH beaconfinder.BeaconFinderApp \
    -s$STDDEV_TOLERANCE_CM -H$SAMPLEMONHISTORY_MS -y$SAMPLEMONMAXY_CM \
    -c$CRICKETDAEMON_IP_ADDR 

