/*    
       This software may be used and distributed according to the terms of
       the GNU General Public License (GPL), incorporated herein by
       reference.  Drivers based on this skeleton fall under the GPL and
       must retain the authorship (implicit copyright) notice.
 
       This program is distributed in the hope that it will be useful, but
       WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
       General Public License for more details.
 */
package beaconfinder;

import cricketdaemon.clientlib.*;
import cricketdaemon.clientlib.data.*;
import cricketdaemon.UnitConv;

import javax.swing.table.*;
import java.util.*;

class TableModel extends AbstractTableModel 
{
    private Object[] beaconData, calcBeaconData, orderedCalcBeaconData;
    public int curBeaconInd;
    private static final long TIMEOUT = 20000;

    private long clockOffset = Long.MIN_VALUE;
    private long clockOffsetLastUpdate   = 0;
    private long CLOCKOFFSETUPDATEPERIOD = 2000;

    public String[] columnNames;

    private String[]
	columnNamesCentimeters = 
    { 
	"name", "age (ms)", "mode (cm)", "mean (cm)", 
	"med (cm)", "max (cm)", "min (cm)", "stdv (cm)"//, "coordinates"
    };

    private String[]
	columnNamesInches = 
    { 
	"name", "age (ms)", "mode (in)", "mean (in)", 
	"med (in)", "max (in)", "min (in)", "stdv (in)"//, "coordinates"
    };
    
    public TableModel()
    {
	super();

	if(BeaconFinderApp.useMetric)
	    columnNames = columnNamesCentimeters;
	else 
	    columnNames = columnNamesInches;
    }

    //private long numOffsetSamples = 0;
    private double offsetAvg = 0;
    public void setClockOffset(long off)
    {
	//System.err.println("PERIOD="+CLOCKOFFSETUPDATEPERIOD+" "+clockOffsetLastUpdate);
	if(System.currentTimeMillis() - clockOffsetLastUpdate > CLOCKOFFSETUPDATEPERIOD) {
	    //offsetAvg = (numOffsetSamples*offsetAvg + off)/(numOffsetSamples + 1);
	    offsetAvg = 0.2*offsetAvg + 0.8*off;
	    //numOffsetSamples++;

	    if((clockOffset == Long.MIN_VALUE) ||
	       ((offsetAvg > 0) && (Math.abs(off) > Math.abs(clockOffset))) ||
	       ((offsetAvg < 0) && (Math.abs(off) < Math.abs(clockOffset))))
		clockOffset = off;
	    
	    clockOffsetLastUpdate = System.currentTimeMillis();

	    //System.err.println("offset: "+off+" current Offset="+clockOffset+" offsetAvg="+offsetAvg);

	}
	

    }

    private long currentTime()
    {
	return System.currentTimeMillis() + clockOffset;
    }

    public void setCalcBeaconData(Object[] a)
    {
	calcBeaconData = a;
    }

    public void setBeaconsHeard(ArrayList beaconsHeard)
    {
	beaconData = makeSet(beaconData, beaconsHeard);
	//for(int i = 0; i < beaconData.length; i++)
	//System.out.println("BeaconFinder::TableModel::setBeaconHeard() "+beaconData[i]);
    }

    public String getColumnName(int col) 
    {
	return columnNames[col];
    }

    public int getRowCount() { 
	if(beaconData == null)
	    return 0;
	else {
	    return beaconData.length; 
	}
    }
    public int getColumnCount() { return columnNames.length; }

    public Object getValueAt(int row, int col) {
	if(beaconData == null)
	    return null;

	if(row >= beaconData.length)
	    return null;

	BeaconRecord c = (BeaconRecord) beaconData[row];

	switch(col) {
	case 8:
	    if(c.pos == null)
		return null;
	    return 
		Util.valueToString(c.pos.x, 0.1)+" "+
		Util.valueToString(c.pos.y, 0.1)+" "+
		Util.valueToString(c.pos.z, 0.1);
	case 0:
	    return c.uniqueName;
	case 1:
	    if(c.lastUpdate == null)
		return null;
	    long diff = currentTime() - c.lastUpdate.longValue();
	    //System.out.println("currentTime="+currentTime()+" "+clockOffset+" "+c.lastUpdate.longValue()+" "+
	    //(currentTime()-clockOffset)+" "+System.currentTimeMillis());
	    return String.valueOf(diff);
	}

	if(c.distStat == null)
	    return null;

	// fixme: very ugly code
	if(BeaconFinderApp.useMetric)
	    return colConversionCentimeters(col, row, c);
	else 
	    return colConversionInches(col, row, c);
    }

    private Object colConversionInches(int col, int row, BeaconRecord c) {
	switch(col) {
	case 2:
	    return Util.valueToString(UnitConv.cricketDistToIN(c.distStat.mode), 0.001);
	case 3:
	    return Util.valueToString(UnitConv.cricketDistToIN(c.distStat.mean), 0.001);
	case 4:
	    return Util.valueToString(UnitConv.cricketDistToIN(c.distStat.median), 0.001);
	case 5:
	    return Util.valueToString(UnitConv.cricketDistToIN(c.distStat.max), 0.001);
	case 6:
	    return Util.valueToString(UnitConv.cricketDistToIN(c.distStat.min), 0.001);
	case 7:
	    // need to re-center offset
	    return Util.valueToString(UnitConv.cricketDistToIN(c.distStat.stddev), 0.001);
	case 9:	    
	    BeaconRecord calcDist = null;
	    if(orderedCalcBeaconData != null)
		calcDist = (BeaconRecord) orderedCalcBeaconData[row];
	    
	    if(calcDist == null)
		return null;
	    else
		return Util.valueToString(UnitConv.cricketDistToIN(calcDist.distStat.dist), 0.001);
	}
	return null;
    }

    private Object colConversionCentimeters(int col, int row, BeaconRecord c) {
	switch(col) {
	case 2:
	    return Util.valueToString(UnitConv.cricketDistToCM(c.distStat.mode), 0.001);
	case 3:
	    return Util.valueToString(UnitConv.cricketDistToCM(c.distStat.mean), 0.001);
	case 4:
	    return Util.valueToString(UnitConv.cricketDistToCM(c.distStat.median), 0.001);
	case 5:
	    return Util.valueToString(UnitConv.cricketDistToCM(c.distStat.max), 0.001);
	case 6:
	    return Util.valueToString(UnitConv.cricketDistToCM(c.distStat.min), 0.001);
	case 7:
	    // need to re-center offset
	    return Util.valueToString(UnitConv.cricketDistToCM(c.distStat.stddev), 0.001);
	case 9:	    
	    BeaconRecord calcDist = null;
	    if(orderedCalcBeaconData != null)
		calcDist = (BeaconRecord) orderedCalcBeaconData[row];
	    
	    if(calcDist == null)
		return null;
	    else
		return Util.valueToString(UnitConv.cricketDistToCM(calcDist.distStat.dist), 0.001);
	}
	return null;
    }

    public Class getColumnClass(int col) { return String.class; }

    public void updateBeaconsHeard() 
    {
	if(beaconData == null) {
	    curBeaconInd = 0;
	    return;
	}

	ArrayList result = new ArrayList();
	long curTime = currentTime();

	for(int i = 0; i < beaconData.length; i++) {
	    BeaconRecord br = (BeaconRecord) beaconData[i];
	    if(!timeOut(br.lastUpdate, curTime))
		result.add(beaconData[i]);

	    //System.out.println("timeout: "+br.lastUpdate+" "+timeOut(br.lastUpdate, curTime));
	}
	//	System.out.println("beaconData foo "+beaconData.length);
	beaconData = result.toArray();

	//	System.out.println("beaconData bar "+beaconData.length);

	Arrays.sort(beaconData);
	curBeaconInd = findClosest(beaconData);
	if(calcBeaconData != null)
	    orderedCalcBeaconData = orderData(beaconData, calcBeaconData);
	else
	    orderedCalcBeaconData = null;

	fireTableDataChanged();
    }

    private boolean timeOut(Long lastUpdate, long curTime)
    {
	if(lastUpdate == null)
	    return false;

	// br.lastUpdate is a hardware clock!!
	// curTime should be adjusted with an offset between 
	// Cricket hardware clock and local clock
	return (lastUpdate.longValue() + TIMEOUT) < curTime;
    }

    private Object[] orderData(Object[] target, Object[] source)
    {
	Object[] result = new Object[target.length];

	for(int i = 0; i < result.length; i++) {
	    Associable t = (Associable) target[i];
	    for(int j = 0; j < source.length; j++) {
		Associable s = (Associable) source[j];
		if(s.isAssociated(t)) {
		    result[i] = source[j];
		    break;
		}
	    }
	}

	return result;
    }


    private Object[] makeSet(Object[] orig, ArrayList more) 
    {
	if(orig != null) {
	    for(int i = 0; i < orig.length; i++) {
		if(!more.contains((BeaconRecord)orig[i]))
		    more.add(orig[i]);
	    }
	}
	return more.toArray();
    }

    private int findClosest(Object[] o)
    {
	double curMin = Double.MAX_VALUE;
	int curMinIndex = -1;
	
	for(int i = 0; i < o.length; i++) {
	    BeaconRecord r = (BeaconRecord) o[i];
	    double comp = r.distStat.mode;
	    if(comp < curMin) {
		curMin = comp;
		curMinIndex = i;
	    }
	}
	
	return curMinIndex;
    }
    
}
