/*    
       This software may be used and distributed according to the terms of
       the GNU General Public License (GPL), incorporated herein by
       reference.  Drivers based on this skeleton fall under the GPL and
       must retain the authorship (implicit copyright) notice.
 
       This program is distributed in the hope that it will be useful, but
       WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
       General Public License for more details.
 */
package cricketdaemon.clientlib;

import cricketdaemon.clientlib.data.CricketData;
import java.util.*;

/**
 * Cricket Applications should implement this interface to receive
 * updates from the <code>cricketdameon</code>.
 *
 * @author Allen Miu
 */
public interface Callback
{
    /**
     * The update from the cricketdaemon is contained in <code>data</code>, 
     * The updated fields are specified in <code>mask</code>.  
     * Beware when multiple
     * threads have registered callbacks with the Broker: the
     * same <code>data</code> gets passed to every thread.  As a result,
     * any modifications made to data may become visible to another
     * thread.  In such case, each thread is responsible for deep
     * cloning the relevant data structure.  Be advised that deep cloning
     * is not implemented for any data strucuture contained within
     * <code>CricketData</code>.
     */
    void callback(CricketData data, BitSet mask); 
}

