/*    
       This software may be used and distributed according to the terms of
       the GNU General Public License (GPL), incorporated herein by
       reference.  Drivers based on this skeleton fall under the GPL and
       must retain the authorship (implicit copyright) notice.
 
       This program is distributed in the hope that it will be useful, but
       WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
       General Public License for more details.
 */
package cricketdaemon.reader;

import cricketdaemon.*;
import cricketdaemon.module.*;
import cricketdaemon.reader.serial.*;
import cricketdaemon.reader.parse.ParseMap;
import java.util.*;
import java.io.*;

/**
 * Retrieves and parses beacon data from the FIRST VERSION
 * of the cricket listeners.
 *
 * @author Bodhi Priyantha, Allen Miu
 */
public class OldSerialPortReader extends SerialPortReader 
{
    // maximum size of location string (Should be set from properties.)
    private static final int MAX_STRING = 100; 

    // start and end of message markers
    private static final byte STX = 2;
    private static final byte ETX = 3;
    private static final byte US  = 31; //unit seperator

    static class OldSerialPortReaderFactory implements PortReaderFactory
    {
	public ModuleThread create(String configFile)
	{
	    return new OldSerialPortReader(configFile);
	}
    }

    static 
    {
	PortReaderCreator.
	    portReaderFactories.put(PortReaderCreator.PATH+
				    "OldSerialPortReader", 
				    new OldSerialPortReaderFactory());
    }

    public OldSerialPortReader(String configFile)
    {
	super(configFile);
    }

    // the space string from the beacon
    private byte[] buf      = new byte[MAX_STRING]; 
    private byte[] distance = new byte[2];  // buffer for the distance


    protected Beacon readData(BitSet mask) 
	throws IOException 
    {
	byte char1, char2, character;
	
	// sync
	char2 = (byte) this.in.read();
	do{
	    char1 = char2;
	    char2 = (byte) this.in.read();
	} while((char1 != ETX)||(char2 != STX));

	Beacon beacon = new Beacon(System.currentTimeMillis());

	// read the location and distance info
	int length = 0; // number of characters in the space string
	int us_count = 0;

	// warning: FIXME: will die if the string length >= MAX_STRING-2 !!!
	while(length < MAX_STRING) {
	    character = (byte) in.read();
	    if (character == US) {
		us_count++;
		if(us_count == 2) // reached the UID field
		    break;
	    }
	    else {
		buf[length] = character;
		length++;
	    }
	}
	byte[] space = new byte[length];
	System.arraycopy(buf, 0, space, 0, length);

	//byte id = in.read(); // reads the ID of the beacon
	distance[0] = (byte) in.read();  // reads higher order byte of distance
	distance[1] = (byte) in.read();  // reads lower order byte of distance

	// now transfer this reading to the buffer
	beacon.insertValue(space, ParseMap.SPACE);
	mask.set(ParseMap.SPACE);
	//	beacon.insertValue(id, ParseMap.BEACONID);
	//      mask.set(ParseMap.BEACONID);
	beacon.insertValue(distance, ParseMap.DISTANCE);
	mask.set(ParseMap.DISTANCE);

	return beacon;
    }

    public static void main(String[] args) 
    {
	OldSerialPortReader opr = 
	    new OldSerialPortReader(CricketDaemon.CONFIGFILE);
	Util.init(CricketDaemon.DEBUGLOGFILE);

	opr.run();
	/*
	opr.start();
	try {
	    opr.join();
	}
	catch (InterruptedException ie) {
	    System.err.println("interrupted!!");
	}
	finally {
	    Util.exit();
	}
	*/
	Util.exit();
    }
}
