/*    
       This software may be used and distributed according to the terms of
       the GNU General Public License (GPL), incorporated herein by
       reference.  Drivers based on this skeleton fall under the GPL and
       must retain the authorship (implicit copyright) notice.
 
       This program is distributed in the hope that it will be useful, but
       WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
       General Public License for more details.
 */
package cricketdaemon.reader.sim;

import cricketdaemon.Util;
import cricketdaemon.clientlib.sim.Coord;
import cricketdaemon.clientlib.sim.SimControl;
//import pathfinder.Waypoint;
import javax.swing.border.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

/**
 * A GUI for the simulation.
 *
 * @author Allen Miu
 */
class GUI extends JPanel
{
    ArrayList beaconCoords;
    ArrayList path;

    protected JFrame f;
    protected Controls cntl;

    Image im;
    Graphics imgc;

    public GUI(String title, int x, int y, SimControl s) 
    {

	scaleX = 1.0;
	scaleY = -1.0;
	
	preferredSize = new Dimension(x,y);
	setBorder(createBorder());

	f = new CustomFrame(title);
	Container container = f.getContentPane();
	container.setLayout(new BorderLayout());
	container.add(this, "West");

	cntl = new Controls((SimControl) s);
	f.addKeyListener(cntl);

	cntl.setBorder(BorderFactory.createLoweredBevelBorder());
	container.add(cntl, "South");

	f.setBackground(Color.white);
	f.validate();
	f.pack();
	f.setVisible(true);

	im = createImage(x,y);
	imgc = (Graphics) im.getGraphics();
	//	imgc.setXORMode(Color.red);
	//	imgc.setColor(Color.blue);
	    
	Util.log("view created and exposed");
    }

    /* public API for program */

    public void setBeacons(ArrayList v)
    {
	beaconCoords = v;
	repaint();
    }

    public void setPath(ArrayList v)
    {
	path = v;
	repaint();
    }

    double PiToDegreeConv = 180.0/Math.PI;

    //    public void drawBeacon
    public void clearBeaconTransmit(double x, double y, double radius)
    {
	int boundX = (int) (transformX(x)-radius-5);
	int boundY = (int) (transformY(y)-radius-5);
	int boundSize = (int) (2*(radius+5));

	imgc.clearRect(boundX, boundY, boundSize, boundSize);
	repaint(boundX, boundY, boundSize, boundSize);
    }

    public void drawBeaconTransmit(double x, double y, double radius)
    {
	int boundX = (int) (transformX(x)-radius);
	int boundY = (int) (transformY(y)-radius);
	int boundSize = (int) (2*radius);

	imgc.setColor(Color.red);
	imgc.drawOval(boundX, boundY, boundSize, boundSize);
	repaint(boundX, boundY, boundSize, boundSize);
    }

    public void clearDevice(double x, double y, double radius)
    {
	int originX = (int) (transformX(x)-radius);
	int originY = (int) (transformY(y)-radius);
	int boxSize = (int)(2*radius); 
	int boundX   = originX-25;
	int boundY   = originY-25;
	int boundSize = boxSize+50;
	imgc.clearRect(boundX, boundY, boundSize, boundSize); 
    }

    public void drawDevice(double x, double y, double or,
			   double vangle, double radius) 
    {
	
	int originX = (int) (transformX(x)-radius);
	int originY = (int) (transformY(y)-radius);
	int boxSize = (int)(2*radius); 
	int start   = (int)((-or-vangle)*PiToDegreeConv);
	int degree  = (int)(2*vangle*PiToDegreeConv) - 1;

	int boundX   = originX-25;
	int boundY   = originY-25;
	int boundSize = boxSize+50;

	int ix = transformX(x);
	int iy = transformY(y);
	
	imgc.setColor(Color.blue);
	_drawLineRadians(ix, iy, radius, or-vangle);
	if(or != 0) 
	    _drawLineRadians(ix, iy, radius, or+vangle);
	imgc.drawArc(originX, originY, boxSize, boxSize, start, degree);
	repaint(boundX, boundY, boundSize, boundSize);
    }

    public void paintDevice(double x, double y, double or,
			    double vangle, double radius) 
    {
	clearDevice(x, y, radius);
	drawDevice(x, y, or, vangle, radius);
    }

    /* private methods for drawing other stuff */
    
    private void _drawBeacons()
    {
	if( beaconCoords != null && imgc != null ) {
	    imgc.setColor(Color.green);		    
	    for(int i = 0; i < beaconCoords.size(); i++) {
		Coord p = (Coord) beaconCoords.get(i);
		
		//int ix = (int)p.x;
		//int iy = (int)p.y;		
		
		int ix = transformX(p.x);
		int iy = transformY(p.y);

		imgc.drawRect(ix, iy, 4, 4);
		imgc.drawString(p.name, ix, iy);
	    }
	}	
	imgc.setColor(Color.black);
    }

    /* transform stuff */
    private double translateX, translateY;
    private double scaleX, scaleY;

    public int transformX(double x)
    {
	return Math.round((float) (x*scaleX+translateX));
    }

    public int transformY(double y)
    {
	return  Math.round((float)(getPreferredSize().getHeight() + y*scaleY+translateY));
    }

    /*
      private void _drawPath()
      {
      if( path != null && imgc != null ) {
      imgc.setColor(Color.red);		    
      for(int i = 0; i < path.size(); i++) {
      Waypoint p = (Waypoint) path.get(i);
      int ix = (int)p.pos.x;
      int iy = (int)p.pos.y;		
      imgc.drawRect(ix, iy, 4, 4);
      imgc.drawString(p.id, ix, iy);
      }
      }	
      imgc.setColor(Color.black);
      }
    */

    private void _drawLineRadians(int ix, int iy, double r, double a)
    {
	int x2 = (int) (r*Math.cos(a));
	int y2 = (int) (r*Math.sin(a));
	imgc.drawLine(ix, iy, ix+x2, iy+y2);
    }

    /* Java AWT stuff */
    public void paintComponent(Graphics g) 
    {
        super.paintComponent(g);
	if( im != null ) {
	    _drawBeacons();
	    //	    _drawPath();
	    g.drawImage(im,0,0,null);
	}
    }

    Dimension preferredSize;
    public Dimension getPreferredSize() 
    {
	return preferredSize;
    }

    private Border createBorder() {
	Border raisedbevel = BorderFactory.createRaisedBevelBorder();
	Border loweredbevel = BorderFactory.createLoweredBevelBorder();
	
	return
	    BorderFactory.createCompoundBorder(raisedbevel, loweredbevel);
    }
}
